﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace BleedOut
{
    public class AsciiChunkHandler : ChunkHandler
    {
        private Action<string> _stringFoundCallback;

        private List<string> _knownStrings = new List<string>();

        public AsciiChunkHandler()
        {

        }

        public AsciiChunkHandler(Action<string> stringFoundCallback)
            : this()
        {
            _stringFoundCallback = stringFoundCallback;
        }

        string[] GetNewStrings(byte[] memory)
        {
            string[] strings = null;

            lock (_knownStrings)
            {
                var str = memory.GetString();

                strings = Regex
                    .Matches(str, @"[\x10-\x7e]{3,}")
                    .OfType<Match>()
                    .Select(x => x.Value)
                    .Distinct()
                    .Where(x => !_knownStrings.Contains(x))
                    .ToArray();

                _knownStrings.AddRange(strings);

                File.AppendAllLines(CliArgs.Instance.AsciiFile, strings);
            }

            return strings;
        }

        public override void ProcessChunk(byte[] chunk)
        {
            var strings = GetNewStrings(chunk);

            if (_stringFoundCallback != null && strings.Any())
            {
                foreach (var s in strings)
                {
                    _stringFoundCallback(s);
                }
            }
        }
    }
}
