﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace BleedOut
{
    public class BinaryChunkHandler : ChunkHandler
    {
        private MD5CryptoServiceProvider _md5 = new MD5CryptoServiceProvider();

        private List<string> _knownChunks = new List<string>();

        private Action<byte[]> _bytesWrittenCallback;

        private string GetHash(byte[] chunk)
        {
            return string.Join("", _md5.ComputeHash(chunk).Select(x => string.Format("{0:X2}", x)));
        }

        public BinaryChunkHandler()
        {

        }

        public BinaryChunkHandler(Action<byte[]> bytesWrittenCallback)
            : this()
        {
            _bytesWrittenCallback = bytesWrittenCallback;
        }

        public override void ProcessChunk(byte[] chunk)
        {
            lock (_knownChunks)
            {
                var hash = GetHash(chunk);

                if (_knownChunks.Contains(hash))
                {
                    return;
                }

                _knownChunks.Add(hash);

                using (var f = File.OpenWrite(CliArgs.Instance.BinaryFile))
                {
                    f.Write(chunk);
                }

                if (_bytesWrittenCallback != null)
                {
                    _bytesWrittenCallback(chunk);
                }
            }
        }
    }
}
